#include "model.h"
#include "scene.h"
#include "texload.h"


CModel::CModel()
{ //atziimeejam visus resursus kaa neizmantotus
    Vertex=NULL;
    UV=NULL;
    Triangle=NULL;
    
    VertexCount=0;
    TriangleCount=0;
    
    DisplayList=0;                
}



CModel::~CModel()
{
    Delete();             
}



bool CModel::LoadFromASCIIFile(char *FileName) //ielaadeesim modeli no vienkaarsha ASCII faila
{
    FILE *fp=fopen(FileName,"rt");
    if(!fp)
    {
        printf("Nevaru ielaadeet ASCII model failu %s - taada faila nav\n",FileName);
        return false;      
    }
    
    fscanf(fp,"%d",&VertexCount); //nolasam cik modelii vertexi
    fscanf(fp,"%d",&TriangleCount); //nolasam cik modelii triistuuri
    
    Vertex=new Vector3d[VertexCount]; //rezerveejam vietu vertexiem un triistuuriem
    UV=new CUV[VertexCount];
    Triangle=new CModelTriangle[TriangleCount];
    
    for(unsigned int i=0;i<VertexCount;i++) //nolasam karta vertexa parametrus no faila
    {
        fscanf(fp,"%f",&Vertex[i].x); //poziicija object space'aa
        fscanf(fp,"%f",&Vertex[i].y);
        fscanf(fp,"%f",&Vertex[i].z);
        fscanf(fp,"%f",&UV[i].U); //UV - textuuru - koordinaates
        fscanf(fp,"%f",&UV[i].V);
    }
    
    for(unsigned int i=0;i<TriangleCount;i++) //nolasam katra triistuura  vertexu indexus
    {
        fscanf(fp,"%d",&Triangle[i].Vert[0]);
        fscanf(fp,"%d",&Triangle[i].Vert[1]);
        fscanf(fp,"%d",&Triangle[i].Vert[2]);
    }
    
    fclose(fp);
    CreateDisplayList(); //uzbuuveejam modelja display listu
    return true;
}



bool CModel::LoadFrom24BitBMPHeightMap(char *FileName) //ielaadeesim modeli kaa zemi no
{                                          //augstuma mapes, kas glabaajas 24bitu bitmapaa
    int Width,Height; //mapes izmeeri
    unsigned char *Data; //pointers, kur saglabaat mapesdatus
    
    if(!Load24BitBMP(FileName,Width,Height,Data))//shii funkcija njem mainiigos peec
    {                            //references - taa savos argumentos atgriez rezultaatu
        printf("Nevaru ielaadeet zemes heightmapu %s\n",FileName);
        return false;
    }
    
    VertexCount=Width*Height; //cik vertexi vajadziigi zemei
    TriangleCount=(Width-1)*(Height-1)*2; //cik treiistuuri vajadziigi zemei
    
    Vertex=new Vector3d[VertexCount];    //atbriivojam vajadziigo vietu
    UV=new CUV[VertexCount];
    Triangle=new CModelTriangle[TriangleCount];
    
    for(int z=0;z<Height;z++) //katram vertexam
        for(int x=0;x<Width;x++)
        {
            int VertNum=z*Width+x;
            Vertex[VertNum].x=x*10.0f;//pieiram atraans vietu
            Vertex[VertNum].z=z*10.0f;
            Vertex[VertNum].y=Data[VertNum*3]; //pieiram attieciigo augstumu no mapes
            UV[VertNum].U=x;//un noraadaam UV koordinaates
            UV[VertNum].V=z;//viena textuuras taile uz vienu augstuma mapes pixeli
        }
    
    int TriNum=0;//kuru triistuuri mees tagad apreekjinaasim
    for(int z=0;z<Height-1;z++)
        for(int x=0;x<Width-1;x++)
        {
            Triangle[TriNum].Vert[0]=(z)*Width+(x); //noraadam attieciigos vertexu indexus
            Triangle[TriNum].Vert[1]=(z+1)*Width+(x);// lai izveidotu zemi no vertexiem
            Triangle[TriNum].Vert[2]=(z)*Width+(x+1);//kuri ielaadeeti no heightmapa
            TriNum++;//paarejam pie naakamaa triistuura
            
            Triangle[TriNum].Vert[0]=(z+1)*Width+(x+1);
            Triangle[TriNum].Vert[1]=(z+1)*Width+(x);
            Triangle[TriNum].Vert[2]=(z)*Width+(x+1);
            TriNum++;
        }
        
    delete[] Data; //nodzeesham textuuras datus - tie vairs nav vajadziigi, vertexi jau zin
                    //savas poziicijas
    CreateDisplayList(); //uztaisam display listu
    return true;
}


bool CModel::Draw()
{
    switch(RenderMode)//ziimejam ar attieciigo metodi, kaada noraadiita RenderModee
        {
            case IMMEDIATE_VALUE:
                return DrawImmediateValue();
            case IMMEDIATE_POINTER:
                return DrawImmediatePointer();
            case VERTEX_ARRAY:
                return DrawVertexArray();
            case DISPLAY_LIST:
                return DrawDisplayList();
        }
    return false;
}



bool CModel::DrawImmediateValue() //ziimeejam izmantojot immediate mode peec veertiibaas
{
    glBegin(GL_TRIANGLES);
    int v1,v2,v3;
    for(unsigned int i=0;i<TriangleCount;i++) //ejam cauri visiem triistuuriem
    {
        v1=Triangle[i].Vert[0];//atrodam katras triistuura virsotnes vertex indexu
        v2=Triangle[i].Vert[1];
        v3=Triangle[i].Vert[2];
        glTexCoord2f(UV[v1].U,UV[v1].V); //ziimeejam katru vertexu peec attieciigaa indexa
        glVertex3f(Vertex[v1].x,Vertex[v1].y,Vertex[v1].z);
        glTexCoord2f(UV[v2].U,UV[v2].V);
        glVertex3f(Vertex[v2].x,Vertex[v2].y,Vertex[v2].z);
        glTexCoord2f(UV[v3].U,UV[v3].V);
        glVertex3f(Vertex[v3].x,Vertex[v3].y,Vertex[v3].z);
    }
    glEnd();
    return true;
}



bool CModel::DrawImmediatePointer() //ziimeejam izmantojot immediate mode ar pointeriem
{
    glBegin(GL_TRIANGLES);
    int v1,v2,v3;
    for(unsigned int i=0;i<TriangleCount;i++) //ejam cauri visiem triistuuriem
    {
        v1=Triangle[i].Vert[0];//atrodam katras triistuura virsotnes vertex indexu
        v2=Triangle[i].Vert[1];
        v3=Triangle[i].Vert[2];
        glTexCoord2fv(&UV[v1].U); //ziimeejam katru vertexu peec attieciigaa indexa
        glVertex3fv(&Vertex[v1].x);
        glTexCoord2fv(&UV[v2].U);
        glVertex3fv(&Vertex[v2].x);
        glTexCoord2fv(&UV[v3].U);
        glVertex3fv(&Vertex[v3].x);
    }
    glEnd();
    return true;
}



bool CModel::DrawVertexArray() //ziimeejam izmantojot vertex array
{
    glEnableClientState( GL_VERTEX_ARRAY ); //iesleedzam vertex array
    glEnableClientState( GL_TEXTURE_COORD_ARRAY ); //iesleedzam vertex array arii prieksh
                                                    //UV koordinaateem
    glVertexPointer(3,GL_FLOAT,0,Vertex); //noraadaam pointeru uz vertexu masiivu un
                                          //paraadam veidu kaa tie tiek glabaati
                                          //t.i. - 3 floati uz vertexu ar 0 baitiem starpaa
    glTexCoordPointer(2,GL_FLOAT,0,UV); //noraadaam pointeru uz UV masiivu un
                                        //paraadam veidu kaa UV tiek glabaati 
                                        //t.i. - 2 floati uz vertexu ar 0 baitiem starpaa
    
    glDrawElements(GL_TRIANGLES,TriangleCount*3,GL_UNSIGNED_INT,Triangle); //ziimeejam ar vertex array
                                            //objektu tips - GL_TRIANGLES
                                            //vertexu indexu skaits - TriangleCount*3
                                            //katra indexa mainiigaa tips - GL_UNSIGNED_INT  
     
    glDisableClientState( GL_TEXTURE_COORD_ARRAY ); //kad esam beigushi ziimeet,
    glDisableClientState( GL_VERTEX_ARRAY ); //izsleedzam abas iesleegtaas virknes
    return true;
}



bool CModel::DrawDisplayList() //ziimeejam ar display list
{
    glCallList(DisplayList); //vienkaarshi izsaucam vajadziigo display listu
    return true;
}


bool CModel::CreateDisplayList() //uzbuuveejam display listu
{
    DisplayList=glGenLists(1); //dabuujam no OGL jaunu DisplayLista ID
    if(!DisplayList) //ja nesanaaca...
    {
        printf("Nevaru uztaisiit display listu\n");
        return false;           
    }
    glNewList(DisplayList,GL_COMPILE); //saakam kompileet uz noraadiito DisplayListu
    //visas taalaakaas darbiiba tiek ierakstiitas, nevis uzreiz izpildiitas
    glBegin(GL_TRIANGLES); //saakam ziimeet triistuurus
    int v1,v2,v3;
    for(unsigned int i=0;i<TriangleCount;i++) //katram triistuurim
    {
        v1=Triangle[i].Vert[0]; //atrodam virsotnju vertexu indexus
        v2=Triangle[i].Vert[1];
        v3=Triangle[i].Vert[2];
        glTexCoord2f(UV[v1].U,UV[v1].V); //ziimeejam attieciigos vertexus
        glVertex3f(Vertex[v1].x,Vertex[v1].y,Vertex[v1].z);
        glTexCoord2f(UV[v2].U,UV[v2].V);
        glVertex3f(Vertex[v2].x,Vertex[v2].y,Vertex[v2].z);
        glTexCoord2f(UV[v3].U,UV[v3].V);
        glVertex3f(Vertex[v3].x,Vertex[v3].y,Vertex[v3].z);
    }
    glEnd();//beidzam ziimeet triistuurus
    glEndList();//beidzam ierakstiit DisplayListu
    //atceries, ka DisplayListaa tiks saglabaatas tikai izsauktaas OpenGL funkcijas
    //piemeeram, shajaa gadiijumaa, tiks saglabaatas daudz glTexCoord2f un glVertex3f ar
    //glBegin saakumaa un glEnd beigaas, nevis viss cikls. mees varam izdariit cik gribam
    //sarezgiitas darbiibas, lai ieguutu veertiibas, ko dot opengl, bet opengl saglabaas
    //tikai sho savu funkciju argumentu galeejaas veertiibas
    return true;   
}



bool CModel::Delete() //atbriivojam aiznjemto atminju
{
    if(Vertex);
        delete[] Vertex;
    Vertex=NULL;
    
    if(UV)
        delete[] UV;
    UV=NULL;
    
    if(Triangle)
        delete[] Triangle;
    Triangle=NULL;
       
    if(DisplayList)
        glDeleteLists(DisplayList,1);
    DisplayList=0;
    
    return true; 
}


